#include "GatanDM3.h"

namespace Gatan
{
    Endian Util::image_endian = UnknownEndian;
    
#ifdef __linux__
    Endian Util::system_endian = LittleEndian;
#elif defined __sgi
    Endian Util::system_endian = BigEndian;
#elif defined __CYGWIN__
    Endian Util::system_endian = LittleEndian;
#else
    Endian Util::system_endian = UnknownEndian;
#endif
    
    TagTable* TagTable::instance = 0;
    
    TagTable::TagTable()
    {
	tags = std::map<const char*, const char*, StrCmp>();
	img_index = 0;
    }
    
    TagTable::~TagTable()
    {	
	for (int i = 0; i < data_list.size(); i++) {
	    if (data_list[i]) {
		delete [] data_list[i];
		data_list[i] = 0;
	    }
	}	
    }
    
    TagTable* TagTable::get_instance()
    {
	if (!instance) {
	    instance = new TagTable();
	}
	return instance;
    }
	
    void TagTable::add(const char* name, const char* value)
    {
	if (!name) {
	    return;
	}
	
	if (!value) {
	    value = "";
	}

	if (strcmp(name, "Dimensions #0") == 0) {
	    x_list.push_back(atoi(value));
	}
	else if (strcmp(name, "Dimensions #1") == 0) {
	    y_list.push_back(atoi(value));
	}
	else if (strcmp(name, "DataType") == 0) {
	    datatype_list.push_back(atoi(value));
	}
	else if (strcmp(name, "ImageIndex") == 0) {
	    set_thumb_index(atoi(value));
	}
	else {
	    tags[name] = value;
	}
    }

    void TagTable::add_data(char* data)
    {
	if (data) {
	    data_list.push_back(data);
	}
    }
    
    const char* TagTable::get_string(const char* name) 
    {
	assert(name);
	assert(strcmp(name, "") != 0);

	const char* result = tags[name];
	if (!result) {
	    result = "";
	}
	return result;
    }

    int TagTable::get_int(const char* name)
    {
	const char* str = get_string(name);
	return atoi(str);
    }
    
    float TagTable::get_float(const char* name)
    {
	const char* str = get_string(name);
	return (float) atof(str);
    }
    
    double TagTable::get_double(const char* name)
    {
	const char* str = get_string(name);
	return atof(str);
    }
    

    void TagTable::dump() const
    {
	typedef std::map<const char*, const char*, StrCmp>::const_iterator MI;

	for (MI p = tags.begin(); p != tags.end(); p++) {
	    printf("%s: %s\n", p->first, p->second);
	}
    }
    
    int TagTable::get_x() const
    {
	return x_list[img_index];
    }
    
    int TagTable::get_y() const
    {
	return y_list[img_index];
    }

    int TagTable::get_datatype() const
    {
	return datatype_list[img_index];
    }
    
    char* TagTable::get_data() const
    {
	return data_list[img_index];
    }
	
    void TagTable::set_thumb_index(int i)
    {
	assert(i == 0 || i == 1);
	if (i == 0) {
	    img_index = 1;
	}
	else {
	    img_index == 0;
	}
	delete [] data_list[i];
	data_list[i] = 0;
    }
	
    
    void swap2(void* data, int n)
    {
	assert(data);
	assert(n >= 0);

	unsigned char s;
	char* d = static_cast<char*>(data);

	for (int i = 0; i < n; i++, d += 2) {
	    s=d[0]; d[0]=d[1]; d[1]=s;
	}
    }


    void swap4(void* data, int n)
    {
	assert(data);
	assert(n >= 0);

	unsigned char s;
	char* d = static_cast<char*>(data);

	for (int i = 0; i < n; i++, d += 4) {
	    s=d[0]; d[0]=d[3]; d[3]=s;
	    s=d[1]; d[1]=d[2]; d[2]=s;
	}
    }

    void swap8(void* data, int n)
    {
	assert(data);
	assert(n >= 0);
    
	unsigned char s;
	char* d = static_cast<char*>(data);

	for (int i = 0; i < n; i++, d += 8) {
	    s=d[0]; d[0]=d[7]; d[7]=s;
	    s=d[1]; d[1]=d[6]; d[6]=s;
	    s=d[2]; d[2]=d[5]; d[5]=s;
	    s=d[3]; d[3]=d[4]; d[4]=s;
	}
    }

    bool Util::is_valid_order(DataFormat f)
    {
	bool valid_byteorder = true;
    
	if (f == TAG_DATA) {
	    if (image_endian != system_endian) {
		valid_byteorder = false;
	    }
	}
	else if (f == NON_TAG) {
	    if (system_endian != BigEndian) {
		valid_byteorder = false;
	    }
	}
	else {
#ifdef DEBUG_DM3
	    printf("Error: invalid data format: '%d'\n", f);
#endif
	    assert(0 && "no such data format");
	}
	
	return valid_byteorder;
    }

    void becomeplatformendian(short* data, DataFormat f)
    {
	if (!Util::is_valid_order(f)) {
	    swap2(data, 1);
	}
    }

    void becomeplatformendian(unsigned short* data, DataFormat f)
    {
	if (!Util::is_valid_order(f)) {
	    swap2(data, 1);
	}
    }

    void becomeplatformendian(int* data, DataFormat f)
    {
	if (!Util::is_valid_order(f)) {
	    swap4(data, 1);
	}
    }

    void becomeplatformendian(unsigned int* data, DataFormat f)
    {
	if (!Util::is_valid_order(f)) {
	    swap4(data, 1);
	}
    }

    void becomeplatformendian(unsigned long* data, DataFormat f)
    {
	if (!Util::is_valid_order(f)) {
	    swap4(data, 1);
	}
    }

    void becomeplatformendian(float* data, DataFormat f)
    {
	if (!Util::is_valid_order(f)) {
	    swap4(data, 1);
	}
    }

    void becomeplatformendian(double* data, DataFormat f)
    {
	if (!Util::is_valid_order(f)) {
	    swap8(data, 1);
	}
    }

    /////////////////////////////////////////////

    TagData::TagData(FILE* data_file, const char* tagname) :
	in(data_file), name(tagname), tag_type(0)
    {
    }


    TagData::~TagData()
    {

    }

    const char* TagData::read_native(bool is_value_stored)
    {
	int sz = typesize();

	char* val_str = new char[32];
    
	if (tag_type == SHORT) {
	    short val = 0;
	    fread(&val, sz, 1, in);
	    becomeplatformendian(&val, TAG_DATA);
	    sprintf(val_str, "%d", val);
	}
	else if (tag_type == USHORT) {
	    unsigned short val = 0;
	    fread(&val, sz, 1, in);
	    becomeplatformendian(&val, TAG_DATA);
	    sprintf(val_str, "%d", val);
	}
	else if (tag_type == LONG) {
	    int val = 0;
	    fread(&val, sz, 1, in);
	    becomeplatformendian(&val, TAG_DATA);
	    sprintf(val_str, "%d", val);
	}
	else if (tag_type == CHAR ||
		 tag_type == OCTET ||
		 tag_type == BOOLEAN) {
	    char val = 0;
	    fread(&val, sz, 1, in);
	    sprintf(val_str, "%d", val);
	}
	else if (tag_type == ULONG) {
	    unsigned long val = 0;
	    fread(&val, sz, 1, in);
	    becomeplatformendian(&val, TAG_DATA);
	    
	    sprintf(val_str, "%d", (int)val);
	}
	else if (tag_type == FLOAT) {
	    float val = 0;
	    fread(&val, sz, 1, in);
	    becomeplatformendian(&val, TAG_DATA);
	    sprintf(val_str, "%f", val);

	}
	else if (tag_type == DOUBLE) {
	    double val = 0;
	    fread(&val, sz, 1, in);
	    becomeplatformendian(&val, TAG_DATA);
	    sprintf(val_str, "%10e", val);
	}
	else {
	    printf("Error: invalid tag type: '%d'\n", tag_type);
	    assert(0 && "no such tag type");
	}

	if (is_value_stored) {
	    TagTable::get_instance()->add(name, val_str);
	}
    
#ifdef DEBUG_DM3
	printf("value = '%s'\n", val_str);
#endif
    
	return val_str;
    }
	

    vector<int> TagData::read_array_types()
    {
	int array_type = 0;
	fread(&array_type, sizeof(array_type), 1, in);
	becomeplatformendian(&array_type, NON_TAG);

#ifdef DEBUG_DM3
	printf("array data type = '%s'\n", to_str((Type)array_type));
#endif
    
	vector<int> item_types;
    
	if (array_type == STRUCT) {
	    item_types = read_struct_types();
	}
	else if (array_type == ARRAY) {
	    item_types = read_array_types();
	    assert(0 && "not sure how to handle this");
	}
	else {
	    item_types.push_back(array_type);
	}

	return item_types;
    }


    // string tag data are stored in unicode
    const char* TagData::read_string(int size)
    {
	if (size <= 0) {
	    return "";
	}
    
	unsigned short* buf = new unsigned short[size];
	char* str = new char[size + 1];
	fread(buf, size*sizeof(unsigned short), 1, in);

	if (!Util::is_valid_order(TAG_DATA)) {
	    swap2(buf, size);
	}

	for (int i = 0; i < size; i++) {
	    str[i] = (char)buf[i];
	}
	str[size] = '\0';
    
	return str;
    }


    int TagData::read_array_data(vector<int> item_types)
    {
	int err = 0;
    
	assert(item_types.size() > 0);
    
	int array_size = 0;
	fread(&array_size, sizeof(array_size), 1, in);
	becomeplatformendian(&array_size, NON_TAG);
	
#ifdef DEBUG_DM3
	printf("array size = %d\n", array_size);
#endif

	int item_size = 0;
	for (int i = 0; i < item_types.size(); i++) {
	    item_size += typesize(item_types[i]);
	}

#ifdef DEBUG_DM3
	printf("%s array item size = %d\n", name, item_size);
#endif

	int buf_size = item_size * array_size;

	if (item_types.size() == 1 &&
	    item_types[0] == USHORT) {
	    const char* val = read_string(array_size);
	    TagTable::get_instance()->add(name, val);

#ifdef DEBUG_DM3
	    printf("value: %s\n", val);
#endif
	}
	else if (strcmp(name, "Data") == 0) {
	    char* data = new char[buf_size];
	    fread(data, buf_size, 1, in);
	    
	    if (!Util::is_valid_order(TAG_DATA)) {
		if (item_size == 2) {	    
		    swap2(data, array_size);
		}
		else if (item_size == 4) {
		    swap4(data, array_size);
		}
		else if (item_size == 8) {
		    swap8(data, array_size);
		}
		else {
		    assert(0 && "cannot handle this type of image data");
		}
	    }

	    TagTable::get_instance()->add_data(data);	    
	}
	else {
	    fseek(in, buf_size, SEEK_CUR);
	}
    
	return err;
    }

    vector<int> TagData::read_struct_types()
    {
#ifdef DEBUG_DM3
	printf("\nreading struct ...\n");
#endif
    
	unsigned long namelength = 0;
	unsigned long nfields = 0;

	fread(&namelength, sizeof(namelength), 1, in);
	becomeplatformendian(&namelength, NON_TAG);
	fread(&nfields, sizeof(nfields), 1, in);
	becomeplatformendian(&nfields, NON_TAG);
    
#ifdef DEBUG_DM3
	printf("namelength = %d\n", namelength);
	printf("num fields = %d\n", nfields);
#endif
    
	vector<int> field_types;
    
	for (int i = 0; i < nfields; i++) {
	    fread(&namelength, sizeof(namelength), 1, in);
	    becomeplatformendian(&namelength, NON_TAG);
#ifdef DEBUG_DM3
	    printf("%dth namelength = %d", i, namelength);
#endif
	    int field_type = 0;
	    fread(&field_type, sizeof(field_type), 1, in);
	    becomeplatformendian(&field_type, NON_TAG);
#ifdef DEBUG_DM3
	    printf(", type = '%s'\n", to_str((Type) field_type));
#endif
	
	    field_types.push_back(field_type);
	}

	return field_types;
    }

    

    int TagData::read_any()
    {
	int err = 0;
    
	fread(&tag_type, sizeof(tag_type), 1, in);
	becomeplatformendian(&tag_type, NON_TAG);
    
#ifdef DEBUG_DM3
	printf("tag type = '%s'\n", to_str((Type)tag_type));
#endif
	
	if (tag_type == ARRAY) {
	    vector<int> item_types = read_array_types();
	    err = read_array_data(item_types);
	}
	else if (tag_type == STRUCT) {
	    vector<int> field_types = read_struct_types();
	
	
	    for (int i = 0; i < field_types.size(); i++) {
		tag_type = field_types[i];
		const char* val = read_native(false);

		char* fieldname = new char[strlen(name) + 6];
		sprintf(fieldname, "%s #%d", name, i);
		TagTable::get_instance()->add(fieldname, val);
	    }
	
#ifdef DEBUG_DM3
	    printf("\n");
#endif    
	}
	else if (tag_type == STRING) {
	    int str_sz = 0;
	    fread(&str_sz, sizeof(str_sz), 1, in);
	    becomeplatformendian(&str_sz, NON_TAG);
	
	    char* val = new char[str_sz + 1];
	    fread(val, str_sz, 1, in);
	    val[str_sz] = '\0';
	    TagTable::get_instance()->add(name, val);
	}
	else {
	    read_native(true);
	}

	return err;
    }

    int TagData::read()
    {
	int err = 0;
    
	const char* DATA_TYPE_MARK = "%%%%";
	const int mark_sz = strlen(DATA_TYPE_MARK);
	char* mark = new char[mark_sz + 1];
	fread(mark, mark_sz, 1, in);
	mark[mark_sz] = '\0';
	
	assert(strcmp(mark, DATA_TYPE_MARK) == 0);
	
	int encoded_types_size = 0;
	fread(&encoded_types_size, sizeof(int), 1, in);
	becomeplatformendian(&encoded_types_size, NON_TAG);
    
#ifdef DEBUG_DM3
	printf("encoded types size = %d\n", encoded_types_size);
#endif
    
	err = read_any();
  

	return err;
    }


    int TagData::typesize() const
    {
	return typesize(tag_type);
    }

    int TagData::typesize(int type) const
    {
	int size = 0;
    
	switch (type) {
	case SHORT:
	case USHORT:
	    size = 2;
	    break;

	case LONG:
	case ULONG:
	case FLOAT:	
	    size = 4;
	    break;
	    
	case DOUBLE:
	    size = 8;
	    break;

	case BOOLEAN:
	case CHAR:
	case OCTET:
	    size = 1;
	    break;

	default:
	    printf("Error: no such type: '%d'\n", type);
	    break;
	}
	
	return size;
    }

    /////////////////////////////////////////////

    TagGroup::TagGroup(FILE* data_file, const char* groupname) :
	in(data_file), name(groupname), entry_id(0)
    {
    }

    TagGroup::~TagGroup()
    {

    }

    int TagGroup::read()
    {
	int err = 0;    
    
	bool issorted = false;
	bool isopen = false;
	int ntags = 0;
    
	fread(&issorted, sizeof(char), 1, in);
	fread(&isopen, sizeof(char), 1, in);
	fread(&ntags, sizeof(ntags), 1, in);
	becomeplatformendian(&ntags, NON_TAG);
    
#ifdef DEBUG_DM3
	printf("issorted = %d, isopen = %d, ntags = %d\n",
	       issorted, isopen, ntags);
#endif
    
	for (int i = 0; i < ntags; i++) {
	    TagEntry tag_entry(in, this);
	    err = tag_entry.read();
	
	    if (err) {
		break;
	    }
	}
    
	return err;
    }

    const char* TagGroup::get_name() const
    {
	assert(strcmp(name, "") != 0);
	return name;
    }

    int TagGroup::get_entry_id()
    {
	int id = entry_id;
	entry_id++;
	return id;
    }

    /////////////////////////////////////////////


    TagEntry::TagEntry(FILE* data_file, TagGroup* parent)
	: in(data_file), parent_group(parent), name(0)
    {
    }

    TagEntry::~TagEntry()
    {

    }


    int TagEntry::read()
    {
	int err = 0;
    
	char tag_type = 0;
    
	fread(&tag_type, sizeof(char), 1, in);
	assert(tag_type == GROUP_TAG || tag_type == DATA_TAG);

	short name_len = 0;
	fread(&name_len, sizeof(short), 1, in);
	becomeplatformendian(&name_len, NON_TAG);

	if (name_len != 0) {    
	    name = new char[name_len + 1];
	    fread(name, name_len, 1, in);
	    name[name_len] = '\0';
	}
	else {
	    const char* parent_name = parent_group->get_name();
	    name_len = strlen(parent_name) + 4;
	    name = new char[name_len + 1];
	    sprintf(name, "%s #%d", parent_name, parent_group->get_entry_id());
	}

#ifdef DEBUG_DM3
	printf("\ntag name: '%s', len: %d, type: '%s'\n",
	       name, name_len, to_str((EntryType)tag_type));
    
#endif    
    
	if (tag_type == DATA_TAG) {
	    TagData tag_data(in, name);
	    err = tag_data.read();
	}
	else if (tag_type == GROUP_TAG) {
	    TagGroup group(in, name);
	    err = group.read();
	}
    
	return err;
    }


    ////////////////////////////////////////////
    
    DM3Reader::DM3Reader(FILE* file) : in(file) 
    {
	filename = "";
	init_vars();
    }
    
    DM3Reader::DM3Reader(const char* filename_) : filename(filename_)
    {
	in = 0;
	init_vars();
    }


    DM3Reader::~DM3Reader()
    {
	if (root_group) {
	    delete root_group;
	    root_group = 0;
	}

	if (in) {
	    fclose(in);
	    in = 0;
	}
    }

    void DM3Reader::init_vars()
    {
	root_group = 0;   
    
	nx = 0;
	ny = 0;
	exp_num = 0;
	data_type = 0;
	camera_x = 0;
	camera_y = 0;
	frame_type = "";
	binning_x = 0;
	binning_y = 0;
    
	zoom = 0;
	exp_time = 0;
	anti_blooming = false;
	mag = 0;    
    }
    
    int DM3Reader::init()
    {
	assert(filename && "image file cannot be NULL");
    
	in = fopen(filename, "r");
	if (!in) {
	    printf("Error: cannot open image '%s'\n", filename);
	    return 1;
	}

	int img_ver = 0;
	int img_size = 0;
	int byte_order = -1;
    
	fread(&img_ver, sizeof(int), 1, in);
	becomeplatformendian(&img_ver, NON_TAG);

	if (img_ver != 3) {
#ifdef DEBUG_DM3
	    printf("Error: wrong DM version. '%s' is not in DM3. init failed.\n", filename);
#endif
	    return 1;
	}
    
	fread(&img_size, sizeof(int), 1, in);
	becomeplatformendian(&img_size, NON_TAG);
    
	fread(&byte_order, sizeof(int), 1, in);
	becomeplatformendian(&byte_order, NON_TAG);
	Gatan::Util::image_endian = (Endian) byte_order;

	if (byte_order != BigEndian && byte_order != LittleEndian) {
#ifdef DEBUG_DM3
	    printf("Error: wrong endian. '%s' is not in DM3. init failed.\n", filename);
#endif
	    return 1;
	}

    
#ifdef DEBUG_DM3
	printf("image version = %d\n", img_ver);
	printf("image size = %d\n", img_size);
	printf("image endian = %s\n", to_str((Endian)byte_order));
#endif
    
	return 0;
    }

    int DM3Reader::read()
    {
	int err = 0;
    
	root_group = new TagGroup(in, "");
	err = root_group->read();
#ifdef DEBUG_DM3
	TagTable::get_instance()->dump();
#endif

	TagTable* tag_table = TagTable::get_instance();
    
	nx = tag_table->get_x();
	ny = tag_table->get_y();
	data_type = tag_table->get_datatype();
	exp_num = tag_table->get_int("Exposure Number");    
	exp_time = tag_table->get_double("Exposure (s)");    
	zoom = tag_table->get_double("Zoom");
	anti_blooming = tag_table->get_int("Antiblooming");
	mag = tag_table->get_double("Indicated Magnification");    
	frame_type = tag_table->get_string("Processing");
	camera_x = tag_table->get_int("Active Size (pixels) #0");
	camera_y = tag_table->get_int("Active Size (pixels) #1");
	binning_x = tag_table->get_int("Binning #0");
	binning_y = tag_table->get_int("Binning #1");

    
	return err;
    }

    int DM3Reader::read_data(float** p_data)
    {
	assert(p_data != 0);
	assert((*p_data) != 0);

	if (nx <= 0 || ny <= 0) {
	    printf("Error: dm3 image hasn't been read yet\n");
	    return 1;
	}

	char *data = TagTable::get_instance()->get_data();
	int n_data = nx * ny;
	
	for (int i = 0; i < n_data; i++) {

	    switch(data_type) {
	    case DataType::SIGNED_INT8_DATA:
		(*p_data)[i] = (float) ((char*) data)[i];
		break;
	    case DataType::UNSIGNED_INT8_DATA:
		(*p_data)[i] = (float) ((unsigned char*) data)[i];
		break;
	    case DataType::SIGNED_INT16_DATA:
		(*p_data)[i] = (float) ((short*) data)[i];
		break;
	    case DataType::UNSIGNED_INT16_DATA:
		(*p_data)[i] = (float) ((unsigned short*) data)[i];
		break;
	    case DataType::SIGNED_INT32_DATA:
		(*p_data)[i] = (float) ((int*) data)[i];
		break;
	    case DataType::UNSIGNED_INT32_DATA:
		(*p_data)[i] = (float) ((unsigned int*) data)[i];
		break;
	    default:
		printf("Error: DM3Reader cannot handle data type '%s'\n",
		       to_str((DataType::Type)data_type));
		delete [] data;
		data = 0;
		return 1;
	    }
	}
	
	delete [] data;
	data = 0;
	
	return 0;
    }
  

    int DM3Reader::get_nx() const
    {    
	return nx;
    }
    
    int DM3Reader::get_ny() const
    {
	return ny;
    }

    int DM3Reader::get_exposure_num() const
    {
	return exp_num;
    }


    int DM3Reader::get_data_type() const
    {
	return data_type;
    }

    int DM3Reader::get_camera_x() const
    {
	return camera_x;
    }

    int DM3Reader::get_camera_y() const
    {
	return camera_y;
    }

    const char* DM3Reader::get_frame_type() const
    {
	return frame_type;
    }

    int DM3Reader::get_binning_x() const 
    {
	return binning_x;
    }

    int DM3Reader::get_binning_y() const 
    {
	return binning_y;
    }

    float DM3Reader::get_zoom_factor() const
    {
	return zoom;
    }

    double DM3Reader::get_exposure_time() const  
    {
	return exp_time;
    }

    bool DM3Reader::get_anti_blooming() const
    {
	return anti_blooming;
    }

    double DM3Reader::get_magnification() const   
    {
	return mag;
    }


    const char* to_str(TagData::Type type)
    {
	const char* str = "unknown";

	switch(type) {
	case TagData::SHORT:
	    str = "short";
	    break;
	case TagData::LONG:
	    str = "long";
	    break;
	case TagData::USHORT:
	    str = "unsigned short";
	    break;
	case TagData::ULONG:
	    str = "unsigned long";
	    break;
	case TagData::FLOAT:
	    str = "float";
	    break;
	case TagData::DOUBLE:
	    str = "double";
	    break;
	case TagData::BOOLEAN:
	    str = "boolean";
	    break;
	case TagData::CHAR:
	    str = "char";
	    break;
	case TagData::OCTET:
	    str = "octet";
	    break;
	case TagData::STRUCT:
	    str = "struct";
	    break;
	case TagData::STRING:
	    str = "string";
	    break;
	case TagData::ARRAY:
	    str = "array";
	    break;
	default:
	    str = "unknown";
	}
	return str;
    }

    const char* to_str(TagEntry::EntryType type)
    {
	const char* str = "unknown";
    
	switch(type) {
	case TagEntry::GROUP_TAG:
	    str = "Group";
	    break;
	case TagEntry::DATA_TAG:
	    str = "Data";
	    break;
	default:
	    str = "unknown";
	}
	return str;
    }

    const char* to_str(Endian byteorder)
    {
	switch(byteorder) {
	case BigEndian:
	    return "BigEndian";
	case LittleEndian:
	    return "LittleEndian";
	}
	
	return "UnknownEndian";
    }
	
    const char* to_str(DataType::Type type)
    {
	switch(type) {
	case DataType::SIGNED_INT16_DATA:
	    return "SIGNED_INT16_DATA";
	case DataType::REAL4_DATA:
	    return "REAL4_DATA";
	case DataType::COMPLEX8_DATA:
	    return "COMPLEX8_DATA";
	case DataType::OBSELETE_DATA:
	    return "OBSELETE_DATA";
	case DataType::PACKED_DATA:
	    return "PACKED_DATA";
	case DataType::UNSIGNED_INT8_DATA:
	    return "UNSIGNED_INT8_DATA";
	case DataType::SIGNED_INT32_DATA:
	    return "SIGNED_INT32_DATA";
	case DataType::RGB_DATA:
	    return "RGB_DATA";
	case DataType::SIGNED_INT8_DATA:
	    return "SIGNED_INT8_DATA";
	case DataType::UNSIGNED_INT16_DATA:
	    return "UNSIGNED_INT16_DATA";
	case DataType::UNSIGNED_INT32_DATA:
	    return "UNSIGNED_INT32_DATA";
	case DataType::REAL8_DATA:
	    return "REAL8_DATA";
	case DataType::COMPLEX16_DATA:
	    return "COMPLEX16_DATA";
	case DataType::BINARY_DATA:
	    return "BINARY_DATA";
	case DataType::RGB_UINT8_0_DATA:
	    return "RGB_UINT8_0_DATA";
	case DataType::RGB_UINT8_1_DATA:
	    return "RGB_UINT8_1_DATA";
	case DataType::RGB_UINT16_DATA:
	    return "RGB_UINT16_DATA";
	case DataType::RGB_FLOAT32_DATA:
	    return "RGB_FLOAT32_DATA";
	case DataType::RGB_FLOAT64_DATA:
	    return "RGB_FLOAT64_DATA";
	case DataType::RGBA_UINT8_0_DATA:
	    return "RGBA_UINT8_0_DATA";
	case DataType::RGBA_UINT8_1_DATA:
	    return "RGBA_UINT8_1_DATA";
	case DataType::RGBA_UINT8_2_DATA:
	    return "RGBA_UINT8_2_DATA";
	case DataType::RGBA_UINT8_3_DATA:
	    return "RGBA_UINT8_3_DATA";
	case DataType::RGBA_UINT16_DATA:
	    return "RGBA_UINT16_DATA";
	case DataType::RGBA_FLOAT32_DATA:
	    return "RGBA_FLOAT32_DATA";
	case DataType::RGBA_FLOAT64_DATA:
	    return "RGBA_FLOAT64_DATA";
	case DataType::POINT2_SINT16_0_DATA:
	    return "POINT2_SINT16_0_DATA";
	case DataType::POINT2_SINT16_1_DATA:
	    return "POINT2_SINT16_1_DATA";
	case DataType::POINT2_SINT32_0_DATA:
	    return "POINT2_SINT32_0_DATA";
	case DataType::POINT2_FLOAT32_0_DATA:
	    return "POINT2_FLOAT32_0_DATA";
	case DataType::RECT_SINT16_1_DATA:
	    return "RECT_SINT16_1_DATA";
	case DataType::RECT_SINT32_1_DATA:
	    return "RECT_SINT32_1_DATA";
	case DataType::RECT_FLOAT32_1_DATA:
	    return "RECT_FLOAT32_1_DATA";
	case DataType::RECT_FLOAT32_0_DATA:
	    return "RECT_FLOAT32_0_DATA";
	case DataType::SIGNED_INT64_DATA:
	    return "SIGNED_INT64_DATA";
	case DataType::UNSIGNED_INT64_DATA:
	    return "UNSIGNED_INT64_DATA";
	default:
	    return "Unknown Type";
	}
	return "Unknown Type";
    } 
}
