// XYData 1/23/00  Steve Ludtke
// This is a simple class for handling 1d data files (2 column)
// data may be evenly or unevenly spaced

#include "XYData.h"

int flcmp(const void *a, const void *b) {
if (*((float *)a)<*((float *)b)) return -1;
if (*((float *)a)<*((float *)b)) return 1;
return 0;
}

XYData::XYData() {
D=(float *) calloc(10,sizeof(float));
N=0;
Na=5;
ylim[0]=1.0e30;
ylim[1]=-1.0e30;
}

XYData::~XYData() {
free(D);
}

// returns end value if outside range
float XYData::yatx(float x) {
int s;
float f;

// If everything is evenly spaced, this should be the right point
s=2*(int)floor((x-D[0])/xs);

// Since we have no guarentees that things are spaced evenly, we may
// have to hunt for the right place to interpolate
if (D[s]>x) {
	while (D[s]>x && s>=0) s-=2;
}
else if (D[s+2]<x) {
	while (D[s+2]<x && s<N*2-2) s+=2;
}

// return 0 if x is out of range
if (s>=N*2 || s<0) return 0;

//if (s>=N*2) return (D[N*2-1]);
//if (s<0) return (D[1]);

f=(x-D[s])/(D[s+2]-D[s]);

//printf("%f %f %f\t %f %f\n",D[s],x,D[s+2],D[s+1],D[s+3]);
return(D[s+1]*(1.0-f)+D[s+3]*f);
}

void XYData::update() {
int i;

qsort(D,N,2*sizeof(float),flcmp);

ylim[0]=1.0e30;
ylim[1]=-1.0e30;

for (i=1; i<N*2; i+=2) {
	if (D[i]>ylim[1]) ylim[1]=D[i];
	if (D[i]<ylim[0]) ylim[0]=D[i];
}
xs=(ylim[1]-ylim[0])/(float)N;


}

int XYData::readFile(const char *fsp) {
FILE *in = fopen(fsp,"r");
char s[256],t[256];

if (!in) return -1;

for (N=0; fgets(s,255,in); N++) {
	if (N==Na) { Na=Na*3/2; D=(float *)realloc(D,Na*2*sizeof(float)); }
	if (s[0]=='#') { N--; continue; }
	if (sscanf(s," %f%[^.0-9-]%f",&D[N*2],t,&D[N*2+1])!=3) break;
}
update();
fclose(in);

return 0;
}

int XYData::writeFile(const char *fsp) {
int i;
FILE *out=fopen(fsp,"w");

if (!out) return -1;

for (i=0; i<N; i++) fprintf(out,"%1.6g\t%1.6g\n",D[i*2],D[i*2+1]);
fclose(out);
return(0);
}

void XYData::setN(int n) { 
	if (n>=Na) { Na=n; D=(float *)realloc(D,Na*2*sizeof(float)); }
	N=n;
}
