#include "EMPif.h"
#include "XYData.h"
#include "mrc.h"
#include "spider.h"
#include "gatan.h"
#include "emim.h"
#include "icos.h"
#include "imagic.h"

#include "PyEuler.h"
#include "PyEMData.h"
#include "PyConversions.h"
#include "PyEMHist.h"
#include "PyVect.h"
#include "PyVolFrag.h"
#include "PyCylinder.h"
#include "PyUtil.h"

#include <boost/python/class_builder.hpp>

#define PY_ARRAY_UNIQUE_SYMBOL Py_Array_pyEM_EMData
#include <Numeric/arrayobject.h>


namespace python = boost::python;

struct FsTypeEnumTypeConverters
    : python::enum_as_int_converters<FsType>
{
};

struct ImageTypeEnumTypeConverters
    : python::enum_as_int_converters<PyEMData::ImageType>
{
};


void add_EMData(python::module_builder& pyEM)
{
    
    python::class_builder<EMData> _EMData(pyEM, "_EMData");    
    python::class_builder<PyEMData> EMData_class(pyEM, "EMData");
    EMData_class.declare_base(_EMData, python::without_downcast);
    

    EMData_class.def(python::constructor<>());

    EMData_class.def(&PyEMData::get_num_array, "get_num_array");
    import_array();
    
    EMData_class.def(&PyEMData::py_copy, "copy");
    EMData_class.def(&PyEMData::py_copyHead, "copyHead");
    EMData_class.def((PyEMData_ptr (PyEMData::*)(int, int, int, int))
		     &PyEMData::py_clip, "clip");
    EMData_class.def((PyEMData_ptr (PyEMData::*)(int, int, int, int, int, int))
		     &PyEMData::py_clip, "clip");

    EMData_class.def(&PyEMData::py_insertClip, "insertClip");
    EMData_class.def(&PyEMData::readMRCArea, "readMRCArea");
    
    EMData_class.def((int (PyEMData::*)(const char*, int, int, int))
		     &PyEMData::readImage, "readImage");

    EMData_class.def((int (PyEMData::*)(const char*, int, int, int, int))
		     &PyEMData::writeImage, "writeImage");
    
    pyEM.def((int (*)(const char*, String&)) fileCount, "fileCount");
    
    EMData_class.def(&PyEMData::readLST, "readLST");
    
    EMData_class.def(&PyEMData::writeLST, "writeLST");

    EMData_class.def(&PyEMData::serialIn, "serialIn");
    EMData_class.def(&PyEMData::serialOut, "serialOut");
    EMData_class.def(&PyEMData::writeDebug, "writeDebug");
    EMData_class.def(&PyEMData::py_calcHist, "calcHist");
    EMData_class.def(&PyEMData::update, "update");
    EMData_class.def(&PyEMData::setPixel, "setPixel");
    EMData_class.def(&PyEMData::Pixel, "Pixel");
    EMData_class.def(&PyEMData::Min, "Min");
    EMData_class.def(&PyEMData::Max, "Max");
    EMData_class.def(&PyEMData::MinLoc, "MinLoc");
    EMData_class.def(&PyEMData::MaxLoc, "MaxLoc");
    EMData_class.def(&PyEMData::Mean, "Mean");
    EMData_class.def(&PyEMData::Sigma, "Sigma");
    EMData_class.def(&PyEMData::SumSq, "SumSq");
    EMData_class.def(&PyEMData::Mean2, "Mean2");
    EMData_class.def(&PyEMData::Sigma2, "Sigma2");
    EMData_class.def(&PyEMData::edgeMean, "edgeMean");
    EMData_class.def(&PyEMData::cirMean, "cirMean");

    EMData_class.def(&PyEMData::py_setParent, "setParent");
    EMData_class.def(&PyEMData::py_Parent, "Parent");
    
    EMData_class.def(&PyEMData::setName, "setName");
    EMData_class.def(&PyEMData::Name, "Name");
    EMData_class.def(&PyEMData::setPath, "setPath");
    EMData_class.def(&PyEMData::Path, "Path");
    EMData_class.def(&PyEMData::setPathN, "setPathN");
    EMData_class.def(&PyEMData::PathN, "PathN");
    EMData_class.def(&PyEMData::xSize, "xSize");
    EMData_class.def(&PyEMData::ySize, "ySize");
    EMData_class.def(&PyEMData::zSize, "zSize");
    EMData_class.def(&PyEMData::setSize, "setSize");
    EMData_class.def(&PyEMData::setRI, "setRI");
    EMData_class.def(&PyEMData::setComplex, "setComplex");
    EMData_class.def(&PyEMData::isComplex, "isComplex");
    EMData_class.def(&PyEMData::setComplexX, "setComplexX");
    EMData_class.def(&PyEMData::isComplexX, "isComplexX");
    EMData_class.def(&PyEMData::hasCTF, "hasCTF");
    EMData_class.def(&PyEMData::hasCTFF, "hasCTFF");
    EMData_class.def(&PyEMData::getFlags, "getFlags");
    EMData_class.def(&PyEMData::aliPeak, "aliPeak");
    EMData_class.def(&PyEMData::setFlipped, "setFlipped");
    EMData_class.def(&PyEMData::isFlipped, "isFlipped");
    EMData_class.def(&PyEMData::Dx, "Dx");
    EMData_class.def(&PyEMData::Dy, "Dy");
    EMData_class.def(&PyEMData::Dz, "Dz");
    
    EMData_class.def(&PyEMData::alt, "alt");
    EMData_class.def(&PyEMData::az, "az");
    EMData_class.def(&PyEMData::phi, "phi");

    EMData_class.def(&PyEMData::py_getEuler, "getEuler");
    EMData_class.def(&PyEMData::py_setRAlign, "setRAlign");
    
    EMData_class.def(&PyEMData::setTAlign, "setTAlign");
    EMData_class.def((void (PyEMData::*)(float, float, float))&PyEMData::setRAlign, "setRAlign");

    EMData_class.def(&PyEMData::setNImg, "setNImg");
    EMData_class.def(&PyEMData::NImg, "NImg");
    EMData_class.def(&PyEMData::getATime, "getATime");
    

    EMData_class.def(&PyEMData::py_getImagicHed, "getImagicHed");
    EMData_class.def(&PyEMData::py_setImagicHed, "setImagicHed");
    EMData_class.def(&PyEMData::py_getMRCHed, "getMRCHed");
    EMData_class.def(&PyEMData::py_getSpiHed, "getSpiHed");

    EMData_class.def(&PyEMData::Busy, "Busy");
    EMData_class.def(&PyEMData::waitReady, "waitReady");
    EMData_class.def(&PyEMData::Wait, "Wait");
    EMData_class.def(&PyEMData::swapout, "swapout");
    EMData_class.def(&PyEMData::swapin, "swapin");
    
    EMData_class.def(&PyEMData::zero, "zero");
    EMData_class.def(&PyEMData::one, "one");

    EMData_class.def(&PyEMData::ri2ap, "ri2ap");
    EMData_class.def(&PyEMData::ap2ri, "ap2ri");
    
    EMData_class.def(&PyEMData::py_doFFT, "doFFT");
    EMData_class.def(&PyEMData::py_doIFT, "doIFT");
    EMData_class.def(&PyEMData::gimmeFFT, "gimmeFFT");
    
    EMData_class.def(&PyEMData::py_doRadon, "doRadon");
    EMData_class.def(&PyEMData::py_vertACF, "vertACF");

    EMData_class.def(&PyEMData::doDWT, "doDWT");
    EMData_class.def(&PyEMData::doIWT, "doIWT");
    EMData_class.def(&PyEMData::DWTFilt, "DWTFilt");
    
    EMData_class.def(&PyEMData::py_convolute, "convolute");
    EMData_class.def(&PyEMData::py_calcCCF, "calcCCF");
    EMData_class.def(&PyEMData::py_calcCCFX, "calcCCFX");
    EMData_class.def(&PyEMData::py_calcMCF, "calcMCF");

    EMData_class.def(&PyEMData::py_littleBigDot, "littleBigDot");
    
    EMData_class.def(&PyEMData::py_transAlign, "transAlign");
    EMData_class.def(&PyEMData::py_rotAlign, "rotAlign");
    EMData_class.def(&PyEMData::py_rotAlignPrecen, "rotAlignPrecen");
    EMData_class.def(&PyEMData::py_rotAlignCH, "rotAlignCH");
    
    EMData_class.def(&PyEMData::py_transAlign3d, "transAlign3d");
    EMData_class.def(&PyEMData::py_RTFAlignRadon, "RTFAlignRadon");
    EMData_class.def(&PyEMData::py_RTAlignRadon, "RTAlignRadon");
    EMData_class.def(&PyEMData::py_RTFAlignSlow, "RTFAlignSlow");
    EMData_class.def(&PyEMData::py_RTFAlignSlowest, "RTFAlignSlowest");
    EMData_class.def(&PyEMData::py_RTFAlign, "RTFAlign");
    EMData_class.def(&PyEMData::py_RTAlign, "RTAlign");

    EMData_class.def(&PyEMData::py_refineAlign, "refineAlign");
    EMData_class.def(&PyEMData::py_makeRFP, "makeRFP");
    EMData_class.def(&PyEMData::py_unwrap, "unwrap");
    EMData_class.def(&PyEMData::py_calcRCF, "calcRCF");

    EMData_class.def(&PyEMData::rotateX, "rotateX");
    EMData_class.def(&PyEMData::rotateAndTranslate, "rotateAndTranslate");
    EMData_class.def(&PyEMData::rotateAndTranslateFast, "rotateAndTranslateFast");
    EMData_class.def(&PyEMData::fastTranslate, "fastTranslate");
    EMData_class.def(&PyEMData::rot180, "rot180");
    EMData_class.def(&PyEMData::cmCenter, "cmCenter");
    EMData_class.def(&PyEMData::applyMask, "applyMask");
    EMData_class.def(&PyEMData::autoMask, "autoMask");
    
    EMData_class.def(&PyEMData::py_applyRadFn, "applyRadFn");
    EMData_class.def(&PyEMData::py_addRadNoise, "addRadNoise");
    
    EMData_class.def(&PyEMData::py_subtract, "subtract");

    
    EMData_class.def(&PyEMData::vFlip, "vFlip");
    EMData_class.def(&PyEMData::hFlip, "hFlip");
    
    EMData_class.def(&PyEMData::py_normalizeTo, "normalizeTo");
    EMData_class.def(&PyEMData::normalizeMax, "normalizeMax");
    EMData_class.def(&PyEMData::edgeNormalize, "edgeNormalize");

    EMData_class.def(&PyEMData::xNormalize, "xNormalize");
    EMData_class.def(&PyEMData::addMaskShell, "addMaskShell");
    
    EMData_class.def(&PyEMData::py_leastSquareNormalizeTo, "leastSquareNormalizeTo");
    EMData_class.def(&PyEMData::py_maskNormalize, "maskNormalize");

    EMData_class.def(&PyEMData::invert, "invert");

    EMData_class.def((void (PyEMData::*)(int, float, float, PyList))
		     &PyEMData::py_calcRadDist, "calcRadDist");
    
    EMData_class.def((void (PyEMData::*)(int, float, float, PyList, float, float))
		     &PyEMData::py_calcRadDist, "calcRadDist");

    EMData_class.def(&PyEMData::py_calcAzDist, "calcAzDist");
    
    EMData_class.def(&PyEMData::radialAverage, "radialAverage");
    EMData_class.def(&PyEMData::radialSubtract, "radialSubtract");
    
    EMData_class.def(&PyEMData::py_dot, "dot");
    
    EMData_class.def(&PyEMData::py_lcmp, "lcmp");
    EMData_class.def(&PyEMData::py_pcmp, "pcmp");  
    EMData_class.def(&PyEMData::py_fscmp, "fscmp");
    EMData_class.def(&PyEMData::py_fsc, "fsc");

    EMData_class.def(&PyEMData::dCenter, "dCenter");
    EMData_class.def(&PyEMData::sigDiff, "sigDiff");
    EMData_class.def(&PyEMData::realFilter, "realFilter");
    EMData_class.def(&PyEMData::filter, "filter");
    
    EMData_class.def(&PyEMData::py_makeAverage, "makeAverage");
    EMData_class.def(&PyEMData::py_makeAverageIter, "makeAverageIter");
    EMData_class.def(&PyEMData::py_makeAverageWT, "makeAverageWT");
    EMData_class.def(&PyEMData::py_makeAverageCTFC, "makeAverageCTFC");
    EMData_class.def(&PyEMData::py_makeAverageCTFCWauto, "makeAverageCTFCWauto");
    EMData_class.def(&PyEMData::py_makeAverageCTFCW, "makeAverageCTFCW");
    
    EMData_class.def(&PyEMData::py_makeMedian, "makeMedian");
    
  
    EMData_class.def(&PyEMData::py_add, "add");
    EMData_class.def(&PyEMData::py_addIncoherent, "addIncoherent");
    EMData_class.def(&PyEMData::py_mult, "mult");

    EMData_class.def(&PyEMData::multConst, "multConst");
    EMData_class.def(&PyEMData::addConst, "addConst");
    
    EMData_class.def(&PyEMData::meanShrink, "meanShrink");
    EMData_class.def(&PyEMData::medianShrink, "medianShrink");
    
    EMData_class.def(&PyEMData::py_commonLines, "commonLines");

    EMData_class.def(&PyEMData::py_commonLinesR, "commonLinesR");
    EMData_class.def(&PyEMData::py_project3d, "project3d");
    EMData_class.def(&PyEMData::py_fftSlice, "fftSlice");

    
    EMData_class.def(&PyEMData::setup4Slice, "setup4Slice");
    EMData_class.def(&PyEMData::py_interpFT3d, "interpFT3d");

    EMData_class.def(&PyEMData::setup4IS, "setup4IS");
    
    EMData_class.def(&PyEMData::py_normSlice, "normSlice");
    EMData_class.def(&PyEMData::py_insertSlice, "insertSlice");
    EMData_class.def(&PyEMData::py_insertSliceWF, "insertSliceWF");

    EMData_class.def(&PyEMData::doneSlice, "doneSlice");
    EMData_class.def(&PyEMData::doneSliceWF, "doneSliceWF");
    
    EMData_class.def(&PyEMData::py_insertSlice3DReal, "insertSlice3DReal");
   
    EMData_class.def(&PyEMData::py_ctfCurve, "ctfCurve");
    EMData_class.def(&PyEMData::py_ctfMap, "ctfMap");

    EMData_class.def(&PyEMData::py_getCTF, "getCTF");
    EMData_class.def(&PyEMData::py_setCTF, "setCTF");

    EMData_class.def(&PyEMData::setCTFF, "setCTFF");
    EMData_class.def(&PyEMData::clearCTF, "clearCTF");
    
    EMData_class.def(&PyEMData::py_CTFCmp, "CTFCmp");
    EMData_class.def(&PyEMData::subNoise, "subNoise");

    EMData_class.def(&PyEMData::py_iftSlice, "iftSlice");

    EMData_class.def((float (PyEMData::*)(int, int, int))&PyEMData::valueAt, "valueAt");
    EMData_class.def((float (PyEMData::*)(int, int))&PyEMData::valueAt, "valueAt");

    EMData_class.def((float (PyEMData::*)(int, int, int))&PyEMData::valueAtSafe, "valueAtSafe");
    EMData_class.def((float (PyEMData::*)(int, int))&PyEMData::valueAtSafe, "valueAtSafe");

    EMData_class.def(&PyEMData::setValueAt, "setValueAt");
    
    EMData_class.def(&PyEMData::py_row, "row");
    EMData_class.def(&PyEMData::py_col, "col");
    EMData_class.def(&PyEMData::py_setRow, "setRow");
    EMData_class.def(&PyEMData::py_setCol, "setCol");
    EMData_class.def(&PyEMData::py_mxMult, "mxMult");
    EMData_class.def(&PyEMData::py_dist, "dist");

    EMData_class.add(python::make_ref(PyEMData::IMAGE_ANY), "IMAGE_ANY");
    EMData_class.add(python::make_ref(PyEMData::IMAGE_SAL), "IMAGE_SAL");
    EMData_class.add(python::make_ref(PyEMData::IMAGE_PGM), "IMAGE_PGM");
    EMData_class.add(python::make_ref(PyEMData::IMAGE_ICOS), "IMAGE_ICOS");
    EMData_class.add(python::make_ref(PyEMData::IMAGE_MRC), "IMAGE_MRC");
    EMData_class.add(python::make_ref(PyEMData::IMAGE_IMAGIC), "IMAGE_IMAGIC");
    EMData_class.add(python::make_ref(PyEMData::IMAGE_IMAGIC3D), "IMAGE_IMAGIC3D");
    EMData_class.add(python::make_ref(PyEMData::IMAGE_EMIM), "IMAGE_EMIM");
    EMData_class.add(python::make_ref(PyEMData::IMAGE_SINGLE_SPIDER), "IMAGE_SINGLE_SPIDER");
    EMData_class.add(python::make_ref(PyEMData::IMAGE_SPIDER), "IMAGE_SPIDER");
    EMData_class.add(python::make_ref(PyEMData::IMAGE_GATAN), "IMAGE_GATAN");
    EMData_class.add(python::make_ref(PyEMData::IMAGE_GATAN3), "IMAGE_GATAN3");
    EMData_class.add(python::make_ref(PyEMData::IMAGE_PIF), "IMAGE_PIF");
    EMData_class.add(python::make_ref(PyEMData::IMAGE_VTK), "IMAGE_VTK");

    EMData_class.def(&PyEMData::py_doc, "doc");

    const char* docstr = "primary class used to access and process EM data. \n\nEach instance of this class will contain a single 'image'. \nThe data may be 1, 2 or 3 dimensional. Supported file formats \ninclude Imagic, MRC, Spider and standard 8 bit formats like \nGIF, TIFF, etc.";

    EMData_class.add(python::make_ref(docstr), "__doc__");

    
    EMData_class.def_raw(EMData_getstate, "__getstate__");
    EMData_class.def_raw(EMData_setstate, "__setstate__");
    EMData_class.getstate_manages_dict();
}


void add_Euler(python::module_builder& pyEM)
{
    python::class_builder<Euler> _Euler(pyEM, "_Euler");
    python::class_builder<PyEuler> Euler_class(pyEM, "Euler");
    Euler_class.declare_base(_Euler, python::without_downcast);
    
    Euler_class.def(python::constructor<>());
    Euler_class.def(python::constructor<float, float, float>());
    Euler_class.def(python::constructor<float, float, float, int>());
    Euler_class.def(python::constructor<float, float, float, float, int>());
    Euler_class.def(python::constructor<PyList>());
    
    Euler_class.def(&PyEuler::init, "init");
    Euler_class.def(&PyEuler::rectify, "rectify");
    Euler_class.def((void (PyEuler::*)(float, float, float, int)) &PyEuler::setAngle, "setAngle");
    Euler_class.def((void (PyEuler::*)(float, float, float, float, int)) &PyEuler::setAngle, "setAngle");
    Euler_class.def((void (PyEuler::*)(PyList)) &PyEuler::py_setAngle, "setAngle"); 
    Euler_class.def(&PyEuler::increment, "increment");
    Euler_class.def(&PyEuler::diff, "diff");
    Euler_class.def(&PyEuler::py_inverse, "inverse");
    Euler_class.def((void (PyEuler::*)(Euler)) &PyEuler::fromLeft, "fromLeft");
    Euler_class.def((void (PyEuler::*)(float, float, float))  &PyEuler::fromLeft, "fromLeft");
    Euler_class.def(&PyEuler::py_EulerMult, "EulerMult");

    Euler_class.def(python::operators<python::op_mul>());
    Euler_class.def(python::operators<python::op_div>());
    Euler_class.def(python::operators<python::op_eq>());
    Euler_class.def(python::operators<python::op_ne>());

    Euler_class.def(&PyEuler::getMaxSymEl, "getMaxSymEl");
    Euler_class.def(&PyEuler::py_setSym, "setSym");
    Euler_class.def(&PyEuler::resetSym, "resetSym");
    Euler_class.def(&PyEuler::py_FirstSym, "FirstSym");
    Euler_class.def(&PyEuler::py_NextSym, "NextSym");
    Euler_class.def(&PyEuler::py_SymN, "SymN");
    Euler_class.def(&PyEuler::valid, "valid");
    
    Euler_class.def(&PyEuler::fromMRC, "fromMRC");
    Euler_class.def(&PyEuler::thetaMRC, "thetaMRC");
    Euler_class.def(&PyEuler::phiMRC, "phiMRC");
    Euler_class.def(&PyEuler::omegaMRC, "omegaMRC");
    
    Euler_class.def(&PyEuler::alt, "alt");
    Euler_class.def(&PyEuler::az, "az");
    Euler_class.def(&PyEuler::phi, "phi");
    Euler_class.def(&PyEuler::alpha, "alpha");
    Euler_class.def(&PyEuler::beta, "beta");
    Euler_class.def(&PyEuler::gamma, "gamma");
    Euler_class.def(&PyEuler::phiSpider, "phiSpider");
    
    Euler_class.def(&PyEuler::thetaSpider, "thetaSpider");
    Euler_class.def(&PyEuler::gammaSpider, "gammaSpider");
    Euler_class.def(&PyEuler::e0, "e0");
    Euler_class.def(&PyEuler::e1, "e1");
    Euler_class.def(&PyEuler::e2, "e2");
    Euler_class.def(&PyEuler::e3, "e3");
    Euler_class.def(&PyEuler::Q, "Q");
    Euler_class.def(&PyEuler::n1, "n1");
    Euler_class.def(&PyEuler::n2, "n2");
    Euler_class.def(&PyEuler::n3, "n3");
    Euler_class.def(&PyEuler::Qsgi, "Qsgi");
    Euler_class.def(&PyEuler::n1sgi, "n1sgi");
    Euler_class.def(&PyEuler::n2sgi, "n2sgi");
    Euler_class.def(&PyEuler::n3sgi, "n3sgi");
    
    Euler_class.def(&PyEuler::py_print, "Print");
    Euler_class.def(&PyEuler::py_printMatrix, "printMatrix");

    Euler_class.def(&PyEuler::py_mat, "mat");
    Euler_class.def(&PyEuler::py_doc, "doc");

    const char* docstr = "This class is used to store either the orientation\n of a slice in the overall volume or the absolute \norientation of a volume data set. This class provides \nroutines for converting euler angles to/from a variety \nof different conventions (Imagic, quaternion, Spider, \netc.), as well as routines for 'multiplication', a \nsymmetry iterator, etc.";
    
    Euler_class.add(python::make_ref(docstr), "__doc__");
    
    Euler_class.def(Euler_getinitargs, "__getinitargs__");
    Euler_class.def_raw(Euler_getstate, "__getstate__");
    Euler_class.def_raw(Euler_setstate, "__setstate__");
    Euler_class.getstate_manages_dict();
}

void add_EMPif(python::module_builder& pyEM)
{
    python::class_builder<EMPif> EMPif_class(pyEM, "EMPif");
    EMPif_class.def(python::constructor<const char*>());
    EMPif_class.def(&EMPif::dump, "dump");
    EMPif_class.def(&EMPif::is_complex_mode, "is_complex_mode");
    EMPif_class.def(&EMPif::is_valid_mode, "is_valid_mode");
    EMPif_class.def(&EMPif::is_right_order, "is_right_order");
    EMPif_class.def(&EMPif::is_valid_header, "is_valid_header");
    
    EMPif_class.def(&EMPif::get_nx, "get_nx");
    EMPif_class.def(&EMPif::get_ny, "get_ny");
    EMPif_class.def(&EMPif::get_nz, "get_nz");

    EMPif_class.def(&EMPif::get_nimg, "get_nimg");
    EMPif_class.def(&EMPif::get_mode, "get_mode");
    EMPif_class.def(&EMPif::get_mode_nbytes, "get_mode_nbytes");
}

void add_EMHist(python::module_builder& pyEM)
{
    python::class_builder<EMHist> _EMHist(pyEM, "_EMHist");
    python::class_builder<PyEMHist> EMHist_class(pyEM, "EMHist");
    EMHist_class.declare_base(_EMHist, python::without_downcast);

    EMHist_class.def(python::constructor<>());
    EMHist_class.def(python::constructor<const char*>());
    
    EMHist_class.def(&PyEMHist::parse, "parse");
    EMHist_class.def(&PyEMHist::setDir, "setDir");
    EMHist_class.def(&PyEMHist::setFinTime, "setFinTime");
    EMHist_class.def(&PyEMHist::py_getDir, "getDir");

    EMHist_class.def_raw(EMHist_getstate, "__getstate__");
    EMHist_class.def_raw(EMHist_setstate, "__setstate__");
    EMHist_class.getstate_manages_dict();
    
}

void add_Quater(python::module_builder& pyEM)
{
    python::class_builder<Quater> _Quater(pyEM, "_Quater");
    python::class_builder<PyQuater> Quater_class(pyEM, "Quater");
    Quater_class.declare_base(_Quater, python::without_downcast);
    
    Quater_class.def(python::constructor<>());
    Quater_class.def(python::constructor<PyEuler&>());
    Quater_class.def(python::constructor<float, float, float, float>());

    Quater_class.def(&PyQuater::get_nx, "get_nx");
    Quater_class.def(&PyQuater::set_nx, "set_nx");
    
    Quater_class.def(&PyQuater::get_ny, "get_ny");
    Quater_class.def(&PyQuater::set_ny, "set_ny");

    Quater_class.def(&PyQuater::get_nz, "get_nz");
    Quater_class.def(&PyQuater::set_nz, "set_nz");
    
    Quater_class.def(&PyQuater::get_Q, "get_Q");
    Quater_class.def(&PyQuater::set_Q, "set_Q");
    
    
    Quater_class.def(&PyQuater::py_fromEuler, "fromEuler");
    Quater_class.def(&PyQuater::py_toEuler, "toEuler");
    Quater_class.def(&PyQuater::py_print, "Print");
    Quater_class.def(&PyQuater::py_doc, "doc");
    
    Quater_class.def(Quater_getinitargs, "__getinitargs__");
    Quater_class.def_raw(Quater_getstate, "__getstate__");
    Quater_class.def_raw(Quater_setstate, "__setstate__");
    Quater_class.getstate_manages_dict();
    
}


void add_Vect(python::module_builder& pyEM) 
{
    python::class_builder<Vect> _Vect(pyEM, "_Vect");
    
    python::class_builder<PyVect> Vect_class(pyEM, "Vect");
    Vect_class.declare_base(_Vect, python::without_downcast);
    
    Vect_class.def(python::constructor<>());
    Vect_class.def(python::constructor<float, float, float>());

    Vect_class.def(&PyVect::normalize, "normalize");

    Vect_class.def( (PyVect (PyVect::*)(PyList)) &PyVect::py_rotate, "rotate");
    Vect_class.def( (PyVect (PyVect::*)(PyEuler&)) &PyVect::py_rotate, "rotate");
    Vect_class.def( (PyVect (PyVect::*)(PyQuater&)) &PyVect::py_rotate, "rotate");
    Vect_class.def( (PyVect (PyVect::*)(PyVect&, float)) &PyVect::py_rotate, "rotate");

    Vect_class.def( (PyVect (PyVect::*)(float, float, float))&PyVect::py_rotate, "rotate");
    Vect_class.def( (PyVect (PyVect::*)(float, float, float, float))&PyVect::py_rotate, "rotate");

    Vect_class.def(&PyVect::py_cross, "cross");
    Vect_class.def(&PyVect::py_getQuater, "getQuater");

    Vect_class.def(&PyVect::X, "X");
    Vect_class.def(&PyVect::Y, "Y");
    Vect_class.def(&PyVect::Z, "Z");
    Vect_class.def(&PyVect::Alt, "Alt");
    Vect_class.def(&PyVect::Az, "Az");
    Vect_class.def(&PyVect::setX, "setX");
    Vect_class.def(&PyVect::setY, "setY");
    Vect_class.def(&PyVect::setZ, "setZ");
    Vect_class.def(&PyVect::setXYZ, "setXYZ");

    Vect_class.def(&PyVect::py_inner, "inner");
    Vect_class.def(&PyVect::py_angle, "angle");

    Vect_class.def(&PyVect::length, "length");
    
    Vect_class.def(&PyVect::py_euler, "euler");
    Vect_class.def((PyEuler (PyVect::*)(PyVect&)) &PyVect::py_eulerTo, "eulerTo");
    Vect_class.def((PyEuler (PyVect::*)(float, float, float))&PyVect::py_eulerTo, "eulerTo");
    
    Vect_class.def((bool (PyVect::*)(PyVect&))&PyVect::py_isNeighbor, "isNeighbor");
    Vect_class.def(&PyVect::py_print, "Print");
    Vect_class.def(&PyVect::py_doc, "doc");
    
    Vect_class.def(python::operators<python::op_mul>(), python::right_operand<float>());
    Vect_class.def(python::operators<python::op_div>(), python::right_operand<float>());
    Vect_class.def(python::operators<python::op_add>(), python::right_operand<float>());
    Vect_class.def(python::operators<python::op_sub>(), python::right_operand<float>());

    Vect_class.def(python::operators<python::op_add>());
    Vect_class.def(python::operators<python::op_sub>());
    Vect_class.def(python::operators<python::op_eq>());

    Vect_class.def(Vect_getinitargs, "__getinitargs__");
    Vect_class.def_raw(Vect_getstate, "__getstate__");
    Vect_class.def_raw(Vect_setstate, "__setstate__");
    Vect_class.getstate_manages_dict();

}

void add_VolFrag(python::module_builder& pyEM)
{
    python::class_builder<VolFrag> _VolFrag(pyEM, "_VolFrag");
    python::class_builder<PyVolFrag> VolFrag_class(pyEM, "VolFrag");
    VolFrag_class.declare_base(_VolFrag, python::without_downcast);
    
    VolFrag_class.def(python::constructor<>());

    VolFrag_class.def(&PyVolFrag::setSize, "setSize");
    VolFrag_class.def(&PyVolFrag::getSize, "getSize");
    VolFrag_class.def(&PyVolFrag::setSizeYZ, "setSizeYZ");
    VolFrag_class.def(&PyVolFrag::setSizeI, "setSizeI");
    VolFrag_class.def(&PyVolFrag::setupIcos, "setupIcos");
    VolFrag_class.def(&PyVolFrag::zero, "zero");
    VolFrag_class.def(&PyVolFrag::validXYZ, "validXYZ");
    VolFrag_class.def(&PyVolFrag::valueAt, "valueAt");
    VolFrag_class.def(&PyVolFrag::setValue, "setValue");
    VolFrag_class.def(&PyVolFrag::addValue, "addValue");
    VolFrag_class.def(&PyVolFrag::multValue, "multValue");
    VolFrag_class.def(&PyVolFrag::py_addFrag, "addFrag");
    VolFrag_class.def(&PyVolFrag::py_writeFile, "writeFile");
    VolFrag_class.def(&PyVolFrag::py_readFile, "readFile");
    VolFrag_class.def(&PyVolFrag::py_doc, "doc");
}

void add_Cylinder(python::module_builder& pyEM)
{

    python::class_builder<Cylinder> _Cylinder(pyEM, "_Cylinder");
    python::class_builder<PyCylinder> Cylinder_class(pyEM, "Cylinder");
    Cylinder_class.declare_base(_Cylinder, python::without_downcast);
    
    Cylinder_class.def(python::constructor<>());
    Cylinder_class.def(python::constructor<const PyVect&, const PyVect&, 
		       float, float, float>());


    Cylinder_class.def(&PyCylinder::py_Center, "Center");
    Cylinder_class.def(&PyCylinder::py_Axis, "Axis");
    Cylinder_class.def(&PyCylinder::py_QuaterForAxis, "QuaterForAxis");
    Cylinder_class.def(&PyCylinder::Xc, "Xc");
    Cylinder_class.def(&PyCylinder::Yc, "Yc");
    Cylinder_class.def(&PyCylinder::Zc, "Zc");
    Cylinder_class.def(&PyCylinder::Alt, "Alt");
    Cylinder_class.def(&PyCylinder::Az, "Az");
    Cylinder_class.def(&PyCylinder::Radius, "Radius");
    
    Cylinder_class.def(&PyCylinder::Length, "Length");
    Cylinder_class.def(&PyCylinder::Weight, "Weight");
    Cylinder_class.def(&PyCylinder::StartID, "StartID");
    Cylinder_class.def(&PyCylinder::EndID, "EndID");
    Cylinder_class.def(&PyCylinder::Label, "Label");
    Cylinder_class.def(&PyCylinder::print, "Print");
    
    Cylinder_class.def(&PyCylinder::toInventor, "toInventor");
    Cylinder_class.def(&PyCylinder::toDejaVu, "toDejaVu");
    Cylinder_class.def(&PyCylinder::toCosec, "toCosec");

    Cylinder_class.def((void (PyCylinder::*)(const PyVect&))
		       &PyCylinder::py_setCenter, "setCenter");
    Cylinder_class.def((void (PyCylinder::*)(float, float, float))
		       &PyCylinder::setCenter, "setCenter");

    Cylinder_class.def((void (PyCylinder::*)(const Vect&))
		       &PyCylinder::setAxis, "setAxis");
    
    Cylinder_class.def((void (PyCylinder::*)(Euler&))
		       &PyCylinder::setAxis, "setAxis");
    

    Cylinder_class.def(&PyCylinder::setRadius, "setRadius");
    Cylinder_class.def(&PyCylinder::setLength, "setLength");
    Cylinder_class.def(&PyCylinder::setWeight, "setWeight");
    
    Cylinder_class.def(&PyCylinder::setStartID, "setStartID");
    Cylinder_class.def(&PyCylinder::setEndID, "setEndID");
    Cylinder_class.def(&PyCylinder::setLabel, "setLabel");
    Cylinder_class.def(&PyCylinder::py_PlaneAngleDiff, "PlaneAngleDiff");
    Cylinder_class.def(&PyCylinder::py_AxisAngleDiff, "AxisAngleDiff");
    Cylinder_class.def(&PyCylinder::Angle2Vect, "Angle2Vect");

    Cylinder_class.def(&PyCylinder::Angle2Point, "Angle2Point");
    Cylinder_class.def(&PyCylinder::py_Angle2Center, "Angle2Center");
    Cylinder_class.def(&PyCylinder::py_NormDistance, "NormDistance");

    Cylinder_class.def(&PyCylinder::py_CenterDistance, "CenterDistance");
    Cylinder_class.def(&PyCylinder::Distance2Point, "Distance2Point");

    Cylinder_class.def(&PyCylinder::py_if_merge, "if_merge");
    Cylinder_class.def(&PyCylinder::py_merge, "merge");
    Cylinder_class.def(&PyCylinder::mirror, "mirror");

    Cylinder_class.def((void (PyCylinder::*)(Euler&, Vect&, Vect&))
		       &PyCylinder::Transform, "Transform");

    Cylinder_class.def((void (PyCylinder::*)(Quater&, Vect&, Vect&))
		       &PyCylinder::Transform, "Transform");
    
    Cylinder_class.def(&PyCylinder::py_doc, "doc");
    
    Cylinder_class.def_raw(Cylinder_getstate, "__getstate__");
    Cylinder_class.def_raw(Cylinder_setstate, "__setstate__");
    Cylinder_class.getstate_manages_dict();    
}



void add_Cylinders(python::module_builder& pyEM)
{
    python::class_builder<Cylinders> _Cylinders(pyEM, "_Cylinders");
    python::class_builder<PyCylinders> Cylinders_class(pyEM, "Cylinders");
    Cylinders_class.declare_base(_Cylinders, python::without_downcast);
    
    Cylinders_class.def(&PyCylinders::size, "size");
    Cylinders_class.def(&PyCylinders::py_copy, "copy");
    Cylinders_class.def(&PyCylinders::py_subset, "subset");
    Cylinders_class.def(&PyCylinders::py_add, "add");
    
    Cylinders_class.def((bool (PyCylinders::*)(const char*))
			&PyCylinders::py_ReadListfile, "ReadListfile");

    Cylinders_class.def((bool (PyCylinders::*)(FILE*))
			&PyCylinders::ReadListfile, "ReadListfile");

    Cylinders_class.def((bool (PyCylinders::*)(const char*))
			&PyCylinders::py_WriteListfile,
			"WriteListfile");
    
    Cylinders_class.def((bool (PyCylinders::*)(FILE*))
			&PyCylinders::WriteListfile, "WriteListfile");
    
    Cylinders_class.def((bool (PyCylinders::*)(const char*))
			&PyCylinders::py_ReadInventor, "ReadInventor");
    
    Cylinders_class.def((bool (PyCylinders::*)(FILE*))
			&PyCylinders::ReadInventor, "ReadInventor");

    Cylinders_class.def((bool (PyCylinders::*)(const char*))
			&PyCylinders::py_WriteInventor, "WriteInventor");

    Cylinders_class.def((bool (PyCylinders::*)(FILE*))
			&PyCylinders::WriteInventor, "WriteInventor");

    
    Cylinders_class.def((bool (PyCylinders::*)(const char*))
			&PyCylinders::py_WriteVRML, "WriteVRML");
    
    Cylinders_class.def((bool (PyCylinders::*)(FILE*))
			&PyCylinders::WriteVRML, "WriteVRML");

    Cylinders_class.def((bool (PyCylinders::*)(const char*))
			&PyCylinders::py_ReadDejaVu, "ReadDejaVu");

    Cylinders_class.def((bool (PyCylinders::*)(FILE*))
			&PyCylinders::ReadDejaVu, "ReadDejaVu");

    Cylinders_class.def((bool (PyCylinders::*)(FILE*, const char*,
					       const char*, const char*))
			&PyCylinders::WriteDejaVu, "WriteDejaVu");

    Cylinders_class.def((bool (PyCylinders::*)(const char*, const char*,
					       const char*, const char*))
			&PyCylinders::WriteDejaVu, "WriteDejaVu");

    Cylinders_class.def((bool (PyCylinders::*)(const char*))
			&PyCylinders::py_ReadCosec, "ReadCosec");

    Cylinders_class.def((bool (PyCylinders::*)(FILE*))
			&PyCylinders::ReadCosec, "ReadCosec");

    
    Cylinders_class.def((bool (PyCylinders::*)(FILE*,
					       const char*, const char*, 
					       const char*, const char*,
					       const char*, const char*))			 
			&PyCylinders::WriteCosec, "WriteCosec");
    
    Cylinders_class.def((bool (PyCylinders::*)(const char*,
					       const char*, const char*,
					       const char*, const char*,
					       const char*, const char*))
			&PyCylinders::WriteCosec, "WriteCosec");
    
    Cylinders_class.def(&PyCylinders::SortLength, "SortLength");
    Cylinders_class.def(&PyCylinders::SortWeight, "SortWeight");
    Cylinders_class.def(&PyCylinders::OrientationDiff, "OrientationDiff");
    Cylinders_class.def(&PyCylinders::mirror, "mirror");

    
    Cylinders_class.def((void (PyCylinders::*)(Quater&, Vect&, Vect&))
			&PyCylinders::Transform, "Transform");
    Cylinders_class.def((void (PyCylinders::*)(Euler&, Vect&, Vect&))
			&PyCylinders::Transform, "Transform");
    
    Cylinders_class.def(&PyCylinders::print, "Print");
    Cylinders_class.def(&PyCylinders::get, "get");
    Cylinders_class.def(&PyCylinders::py_doc, "doc");
    
    Cylinders_class.def_raw(Cylinders_getstate, "__getstate__");
    Cylinders_class.def_raw(Cylinders_setstate, "__setstate__");
    Cylinders_class.getstate_manages_dict();    
}

void add_ObjectStat(python::module_builder& pyEM)
{
    python::class_builder<ObjectStat> _ObjectStat(pyEM, "_ObjectStat");
    python::class_builder<PyObjectStat> ObjectStat_class(pyEM, "ObjectStat");
    ObjectStat_class.declare_base(_ObjectStat, python::without_downcast);
    
    ObjectStat_class.def((void (PyObjectStat::*)(PyEMData&, const PyList&))
			 &PyObjectStat::py_SetObject, "SetObject");
    
    ObjectStat_class.def((void (PyObjectStat::*)(PyEMData&))
			 &PyObjectStat::py_SetObject, "SetObject");
    
    
    ObjectStat_class.def(&PyObjectStat::py_EigenSortTo, "EigenSortTo");
    ObjectStat_class.def(&PyObjectStat::FitEllipsoid, "FitEllipsoid");
    ObjectStat_class.def(&PyObjectStat::Norm, "Norm");
    ObjectStat_class.def(&PyObjectStat::ComputeLongAxis, "ComputeLongAxis");
    ObjectStat_class.def(&PyObjectStat::print, "Print");
    ObjectStat_class.def(&PyObjectStat::py_doc, "doc");
    
    ObjectStat_class.def_raw(ObjectStat_getstate, "__getstate__");
    ObjectStat_class.def_raw(ObjectStat_setstate, "__setstate__");
    ObjectStat_class.getstate_manages_dict();        
}


void add_Cylinder_related(python::module_builder& pyEM)
{
    python::class_builder<Helix> Helix_class(pyEM, "Helix");
    python::class_builder<HHH> HHH_class(pyEM, "HHH");
    python::class_builder<CylLenComp> CylLenComp_class(pyEM, "CylLenComp");
    python::class_builder<CylWeightComp> CylWeightComp_class(pyEM, "CylWeightComp");
    
    //pyEM.def(radprofile, "radprofile");
}


void add_util(python::module_builder& pyEM)
{
    pyEM.add(python::make_ref(PI), "PI");

    pyEM.add(python::make_ref(RTOD), "RTOD");
    pyEM.add(python::make_ref(MAXPATHLEN), "MAXPATHLEN");
    pyEM.add(python::make_ref(FS_TEXT), "FS_TEXT");
    pyEM.add(python::make_ref(FS_IMAGIC), "FS_IMAGIC");

    pyEM.add(python::make_ref(FS_DONE), "FS_DONE");
    pyEM.add(python::make_ref(FS_LST), "FS_LST");
    pyEM.add(python::make_ref(FS_NULL), "FS_NULL");
    pyEM.add(python::make_ref(FS_BIN), "FS_BIN");
    pyEM.add(python::make_ref(LOG_MISC), "LOG_MISC");
  
    pyEM.add(python::make_ref(LOG_BEGIN), "LOG_BEGIN");
    pyEM.add(python::make_ref(LOG_END), "LOG_END");
    pyEM.add(python::make_ref(LOG_INFILE), "LOG_INFILE");
    pyEM.add(python::make_ref(LOG_OUTFILE), "LOG_OUTFILE");
    pyEM.add(python::make_ref(LOG_REFFILE), "LOG_REFFILE");
    pyEM.add(python::make_ref(LOG_EXTIN), "LOG_EXTIN");
    pyEM.add(python::make_ref(LOG_EXTOUT), "LOG_EXTOUT");
    pyEM.add(python::make_ref(LOG_EXTREF), "LOG_EXTREF");
    pyEM.add(python::make_ref(LOG_ERROR), "LOG_ERROR");

    python::class_builder<ANG> ANG_class(pyEM, "ANG");
    python::class_builder<euler> euler_class(pyEM, "euler");

    pyEM.def(tmpnam2,"tmpnam2");
    pyEM.def(py_resolv,"resolv");
    pyEM.def(sock_open,"sock_open");
    
    pyEM.def(writesock,"writesock");
    pyEM.def(FSsend,"FSsend");
    pyEM.def((float (*)(float, float)) maxf, "maxf");
    pyEM.def((float (*)(float, float, float)) maxf, "maxf");
    pyEM.def((float (*)(float, float, float, float)) maxf, "maxf");

    pyEM.def((float (*)(float, float, float)) minf,"minf");
    pyEM.def((float (*)(float, float, float, float)) minf,"minf");
    pyEM.def(floor2,"floor2");
    pyEM.def(py_round,"round");
    pyEM.def(hypot2,"hypot2");
    pyEM.def(hypot3,"hypot3");

    pyEM.def(py_addHTMLSection,"addHTMLSection");
    pyEM.def((void (*)(const char*, int))py_error,"error");
    pyEM.def((void (*)(int, const char*))py_error,"error");

    pyEM.def(py_memmem,"memmem");
    pyEM.def(file_lock_wait,"file_lock_wait");
    pyEM.def(file_lock,"file_lock");
    pyEM.def(file_unlock,"file_unlock");
    pyEM.def(DFopenr,"DFopenr");
    pyEM.def(DFopenw,"DFopenw");
    pyEM.def(DFopena,"DFopena");

    pyEM.def(py_DFdir,"DFdir");
    pyEM.def(py_EMDIR,"EMDIR");
    pyEM.def(py_LOG,"LOG");
    pyEM.def(py_LOGbegin,"LOGbegin");
    pyEM.def(py_LOGerror,"LOGerror");
    pyEM.def(py_appinit,"appinit");
    pyEM.def(py_appusage,"appusage");
    pyEM.def(frand,"frand");
    pyEM.def(grand,"grand");
    pyEM.def(cyc,"cyc");
    
    pyEM.def(py_interpolate,"interpolate");
    pyEM.def(bilin,"bilin");

    pyEM.def(p5peak,"p5peak");
    pyEM.def(agauss,"agauss");
    pyEM.def(SQR,"SQR");

    pyEM.def(py_roteulertb,"roteulertb");
    
    pyEM.def(npfa,"npfa");
    pyEM.def(isvalidfft,"isvalidfft");
    pyEM.def(bestfftsize,"bestfftsize");
    
    pyEM.def((void (*)(PyList, PyList, int, const char*)) py_save_data,"save_data");
    pyEM.def((void (*)(float, float, PyList, int, const char*)) py_save_data,"save_data");

    pyEM.def(py_lsqfit,"lsqfit");
    pyEM.def(py_run,"run");
    pyEM.def(check_file_type,"check_file_type");
}


void add_image_formats(python::module_builder& pyEM)
{    
    python::class_builder<mrcH> mrcH_class(pyEM, "mrcH");
    mrcH_class.add(python::make_ref(MODE_char), "MODE_char");
    mrcH_class.add(python::make_ref(MODE_short), "MODE_short");
    mrcH_class.add(python::make_ref(MODE_float), "MODE_float");
    mrcH_class.add(python::make_ref(MODE_short_COMPLEX), "MODE_short_COMPLEX");
    mrcH_class.add(python::make_ref(MODE_float_COMPLEX), "MODE_float_COMPLEX");

    python::class_builder<SPIDERH> SPIDERH_class(pyEM, "SPIDERH");
    python::class_builder<PifFileHeader> PifFileHeader_class(pyEM, "PifFileHeader");
    python::class_builder<PifImageHeader> PifImageHeader_class(pyEM, "PifImageHeader");
    python::class_builder<imagicH> imagicH_class(pyEM, "imagicH");
    python::class_builder<icosH> icosH_class(pyEM, "icosH");
    python::class_builder<gatanH> gatanH_class(pyEM, "gatanH");
    python::class_builder<emimH1> emimH1_class(pyEM, "emimH1");
    python::class_builder<emimH2> emimH2_class(pyEM, "emimH2");
    
}

void add_XYData(python::module_builder& pyEM)
{
    python::class_builder<XYData> XYData_class(pyEM, "XYData");    
    XYData_class.def(python::constructor<>());

    XYData_class.def(&XYData::readFile, "readFile");
    XYData_class.def(&XYData::writeFile, "writeFile");
    XYData_class.def(&XYData::update, "update");
    XYData_class.def(&XYData::x, "x");
    XYData_class.def(&XYData::y, "y");
    XYData_class.def(&XYData::n, "n");
    XYData_class.def(&XYData::yatx, "yatx");
    XYData_class.def(&XYData::miny, "miny");
    XYData_class.def(&XYData::maxy, "maxy");
   
    XYData_class.def(&XYData::validx, "validx");
    XYData_class.def(&XYData::setN, "setN");    
}


void add_simple_types(python::module_builder& pyEM)
{    
    python::class_builder<Float> float_class(pyEM, "Float");
    float_class.def(python::constructor<>());
    float_class.def(python::constructor<float>());
    float_class.def_getter(&Float::val, "val");
    float_class.def_setter(&Float::val, "val");


    python::class_builder<String> string_class(pyEM, "String");
    string_class.def(python::constructor<>());
    string_class.def(python::constructor<const char*>());
    string_class.def_getter(&String::val, "val");
    string_class.def_setter(&String::val, "val");
    
}

void add_doc(python::module_builder& pyEM)
{
    const char* docstr = "EMAN classes and routines for image/volume processing in single particle reconstructions.\n\nThis exports:\n  - pyEM.EMData is the primary class to process EM data.\n  - pyEM.Euler is used to store the orientation of a slice. \n  - pyEM.Quater  \n  - pyEM.Vect \n  - pyEM.XYData \n  - pyEM.EMPif is used to process PIF format image. \n  - pyEM.EMHist  \n  - pyEM.VolFrag \n  - pyEM.Cylinder \n  - pyEM.Cylinders \n  - pyEM.ObjectStat \n";
 
    
    pyEM.add(python::make_ref(docstr), "__doc__");
}

BOOST_PYTHON_MODULE_INIT(libpyEM)
{
    python::module_builder pyEM("libpyEM");
    
    add_simple_types(pyEM);    
    add_image_formats(pyEM);    
    add_Euler(pyEM);
    add_Quater(pyEM);
    add_Vect(pyEM);    
    add_XYData(pyEM);
    add_EMPif(pyEM);
    add_EMHist(pyEM);
    add_VolFrag(pyEM);
    add_util(pyEM);
    add_Cylinder(pyEM);
    add_Cylinders(pyEM);
    add_ObjectStat(pyEM);
    add_Cylinder_related(pyEM);
    add_EMData(pyEM);
    add_doc(pyEM);
}
