#include "PyUtil.h"
#include "util.h"


unsigned long py_resolv(const char *host)
{
    return resolv(const_cast<char*>(host));
}

const char* py_DFdir()
{
    return static_cast<const char*>(DFdir());
}

const char* py_EMDIR()
{
    return static_cast<const char*>(EMDIR());
}

void py_LOG(int ref, const char* file, int mode, const char* text)
{
    LOG(ref, const_cast<char*>(file), mode, const_cast<char*>(text));
}


int py_LOGbegin(PyList arg_list, int ppid)
{
    int argc = arg_list.size();
    const char** argv = new const char*[argc];
    list2array<const char*>(arg_list, argv);    

    int result = LOGbegin(argc, const_cast<char**>(argv), ppid);

    delete [] argv;
    argv = 0;
    return result;
}

void py_appinit(PyList arg_list)
{
    int argc = arg_list.size();
    const char** argv = new const char*[argc];
    list2array<const char*>(arg_list, argv);    
    appinit(argc, const_cast<char**>(argv));
    delete [] argv;
    argv = 0;
}


float py_interpolate(float xl, int n, PyList x_list, PyList y_list)
{
    float* x_array = new float[x_list.size()];
    float* y_array = new float[y_list.size()];

    list2array<float>(x_list, x_array);
    list2array<float>(y_list, y_array);

    float result = interpolate(xl, n, x_array, y_array);

    array2list<float>(x_array, x_list);
    array2list<float>(y_array, y_list);
    
    delete [] x_array;
    x_array = 0;
    delete [] y_array;
    y_array = 0;
    
    return result;
}



void py_roteulertb(Float alt, Float az, Float phi, float dx, float dy)
{
    roteulertb(&alt.val, &az.val, &phi.val, dx, dy);
}



void py_save_data(PyList x_list, PyList y_list, int n, const char* fsp)
{
#ifdef DEBUG
    assert(x_list.size() >= n);
    assert(y_list.size() >= n);
#endif
    
    float* x_array = new float[x_list.size()];
    float* y_array = new float[y_list.size()];

    list2array<float>(x_list, x_array);
    list2array<float>(y_list, y_array);

    save_data(x_array, y_array, n, fsp);

    array2list<float>(x_array, x_list);
    array2list<float>(x_array, x_list);

    delete [] x_array;
    x_array = 0;
    delete [] y_array;
    y_array = 0;
}

void py_save_data(float x0, float dx, PyList y_list, int n, const char* fsp)
{
#ifdef DEBUG
    assert(y_list.size() >= n);
#endif

    float* y_array = new float[y_list.size()];
    list2array<float>(y_list, y_array);

    save_data(x0, dx, y_array, n, fsp);

    array2list<float>(y_array, y_list);
    delete [] y_array;
    y_array = 0;
}


void py_lsqfit(int n, PyList dx_list, PyList dy_list, Float m,
	       Float b, int ign0)
{
    float* dx_array = new float[dx_list.size()];
    float* dy_array = new float[dy_list.size()];

    list2array<float>(dx_list, dx_array);
    list2array<float>(dy_list, dy_array);

    lsqfit(n, dx_array, dy_array, &m.val, &b.val, ign0);

    array2list<float>(dx_array, dx_list);
    array2list<float>(dy_array, dy_list);

    delete [] dx_array;
    dx_array = 0;
    delete [] dy_array;
    dy_array = 0;
}

void py_error(const char* msg, int pri)
{
    error(const_cast<char*>(msg), pri);
}

void py_error(int pri, const char* msg)
{
    error(pri, const_cast<char*>(msg));
}

void py_addHTMLSection(const char* sourcefile, FILE* out, const char* secid)
{
    addHTMLSection(const_cast<char*>(sourcefile), out, const_cast<char*>(secid));
}

const char* py_memmem(const char *haystack, int hsize, const char *needle, int nsize)
{
    return static_cast<const char*>(memmem(const_cast<char*>(haystack), hsize, 
					   const_cast<char*>(needle), nsize));
}

void  py_appusage(const char *argv0)
{
    appusage(const_cast<char*>(argv0));
}

int  py_run(const char *s)
{
    return run(const_cast<char*>(s));
}


void py_LOGerror(int ref, int err, const char *text)
{
    LOGerror(ref, err, const_cast<char*>(text));
}

int py_round(float x)
{
    int n = round(x);
    return n;
}
