#ifndef _pyvect_h_
#define _pyvect_h_

#include "Vect.h"
#include "PyEuler.h"

namespace python = boost::python;

class PyQuater : public Quater {
public:
    PyQuater();
    PyQuater(PyEuler& e); 
    PyQuater(float x, float y, float z, float q);
    void py_fromEuler(PyEuler& e);
    PyEuler py_toEuler();
    void py_print(const char* id=NULL, FILE* out=stdout);

    float get_nx() const;
    float get_ny() const;
    float get_nz() const;
    float get_Q() const;

    void set_nx(float x);
    void set_ny(float y);
    void set_nz(float z);
    void set_Q(float q);
    static const char* py_doc(const char* funcname);
};



class PyVect : public Vect {
public:
    PyVect();
    PyVect(float xx, float yy, float zz);

    PyVect py_rotate(PyList m);
    PyVect py_rotate(PyEuler& ang);
    PyVect py_rotate(PyQuater& q);
    PyVect py_rotate(PyVect& p, float q);
    PyVect py_rotate(float alt, float az, float phi);
    PyVect py_rotate(float nx, float ny, float nz, float q);
    
    PyVect py_cross(PyVect& p);
    PyQuater py_getQuater();
    float py_inner(PyVect& p);
    float py_angle(PyVect& p, PyVect y = PyVect());

    PyEuler py_euler();
    PyEuler py_eulerTo(PyVect& p);
    PyEuler py_eulerTo(float x, float y, float z);
    bool py_isNeighbor(PyVect& p);
    void py_print(const char* id=NULL, FILE* out=stdout);
    static const char* py_doc(const char* funcname);
    
    friend PyVect operator * (const PyVect& lhs, float rhs);
    friend PyVect operator / (const PyVect& lhs, float rhs);
    friend PyVect operator + (const PyVect& lhs, float rhs);
    friend PyVect operator + (const PyVect& lhs, const PyVect& rhs);
    friend PyVect operator - (const PyVect& lhs, float rhs);
    friend PyVect operator - (const PyVect& lhs, const PyVect& rhs);
    friend bool operator == (const PyVect& lhs, const PyVect& rhs);

};

python::ref Quater_getinitargs(PyQuater& quater);
python::ref Quater_getstate(const python::tuple& args,
			    const python::dictionary& keywords);
PyObject* Quater_setstate(const python::tuple& args,
			  const python::dictionary& keywords);

python::ref Vect_getinitargs(PyVect& euler);
python::ref Vect_getstate(const python::tuple& args,
			  const python::dictionary& keywords);
PyObject* Vect_setstate(const python::tuple& args,
			const python::dictionary& keywords);

#endif
