#include "PyVolFrag.h"
#include <boost/python/objects.hpp>

namespace python = boost::python;

PyVolFrag::PyVolFrag() : VolFrag() {}

void PyVolFrag::py_addFrag(PyVolFrag& src)
{
    addFrag(&src);
}

int PyVolFrag::py_writeFile(const char *file)
{
    return writeFile(const_cast<char*>(file));
}

int PyVolFrag::py_readFile(const char *file)
{
    return readFile(const_cast<char*>(file));
}

const char* PyVolFrag::py_doc(const char* funcname)
{
   if (!funcname) {
	return "";
    }

    static bool has_built = false;
    static python::dictionary dict = python::dictionary();
    
    if (!has_built) {
	dict["VolFrag"] = "VolFrag()\n  construct an empty VolFrag instance";
	dict["setSize"] = "setSize(size)\n  sets object size";
	dict["getSize"] = "getSize()";
	dict["setSizeYZ"] = "setSizeYZ(nx,y, z)";
	dict["setSizeI"] = "setSizeI(nx, i)";
	dict["setupIcos"] = "setupIcos()";
	dict["zero"] = "zero() \n  zero all elements";	
	dict["validXYZ"] = "int validXYZ(x, y, z)";	
	dict["valueAt"] = "float valueAt(x, y, z)";
	dict["setValue"] = "setValue(x, y, z, val)";
	dict["addValue"] = "addValue(x, y, z, val)";
	dict["multValue"] = "multValue(x, y, z, val)";
	dict["addFrag"] = "addFrag(VolFrag src)\n  only within the intersection of the 2 frags";
	dict["writeFile"] = "writeFile(filename)\n  return 0 on success; not 0 on failure";
	dict["readFile"] = "readFile(filename)\n  return 0 on success; not 0 on failure";
	
	has_built = true;
    }
    
    const char* fdoc = BOOST_PYTHON_CONVERSION::from_python(dict.get_item(funcname).get(),
							    python::type<const char*>());
    if (!fdoc) {
	fdoc = "Error: not defined";
    }
    return fdoc;
}
