// matrixclass.C 10/19/2000  Steven Ludtke

//N matrixclass
//F This program will take a distance matrix and a pre-embedded space
//T X
//1
//P <input file>	Source matrix file from classesbymra
//P <reference file>	Already embedded locations
//P <output file>	Destination file (ascii)
//P [fixq=<val>]	Output with a fixed quality factor rather than the calculated one
//P [classes]	This will classify the particles and output a set of cls files like classesbymra.
//U matrixclass matrix.dat groel.loc class.loc
//D This program is still experimental.
//D This program is used in conjunction with matrixembed and the 'matrix' option of
//E classesbymra. It is used to assign embedded locations to a set of points based on
//E distances to a set of already embedded points. The typical sequence would be as 
//E follows:
//D <UL>
//E <LI>classesbymra proj.hed proj.hed matrix</LI>
//E <LI>matrixembed matrix.dat proj.loc euler=proj.hed</LI>
//E <LI>classesbymra start.hed proj.hed matrix</LI>
//E <LI>matrixclass matrix.dat proj.loc start.loc</LI>
//E </UL>

#include <stdio.h>
#include <math.h>
#include "util.h"
#include <string.h>
#include "EMData.h"

float stepdown(EMData *ps,EMData *ds,EMData *refpos);
float sigma(EMData *ps,EMData *ds,EMData *refpos);
float stepdown(EMData *ps,EMData *ds,EMData *refpos,float last = 0);
EMData *dsdp(EMData *ps,EMData *ds,EMData *refpos);

int sphere=0;
float spherer=0.5;

int main(int argc,char *argv[]) {
int i,j,k,l,DIM=3,classes=0;
int nr,np;	// nr is number of references, np is number of classified points
float *eul=NULL;	// These are the per-image euler angles
float fixq=9999.0;
char s[1024];

appinit(argc,argv);

if (argc<4) appusage(argv[0]);

for (i=4; i<argc; i++) {	
	if (strncmp(argv[i],"classes",7)==0) classes=1;
	else if (strncmp(argv[i],"fixq=",5)==0) fixq=atof(&argv[i][5]);
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}

EMData mx,*refpos,*pos;	// mx is the distance matrix, pos contains DIM x np positions

mx.readImage(argv[1],0);
nr=mx.xSize();
np=mx.ySize();
eul=(float *)malloc(np*3*sizeof(float));

// Now we read the pre-embedded reference file
FILE *in=fopen(argv[2],"r");
fgets(s,79,in);
// First line of the file should contain a magic number and the dimensionality of the embedded space
if (sscanf(s,"#MTX %d",&DIM)!=1) error(ERR_EXIT,"Reference file must begin with '#MTX <DIM>'.");
refpos=new EMData;
refpos->setSize(DIM,nr);
float *rpd = refpos->getData();
for (i=0; i<nr; i++) {
	if (!fgets(s,1023,in)) {
		printf("Only %d elements in reference file, but matrix contains %d.\n",i,nr);
		exit(1);
	}
	k=0;

	// DIM embedded positions
	for (j=0; j<DIM; j++) {
		sscanf(s+k," %f%n",rpd+i*DIM+j,&l);
		k+=l;
	}

	// Quality factor (ignore)
	float tmp;
	sscanf(s+k," %f%n",&tmp,&l);
	k+=l;

	// Euler angles
	for (j=0; j<3; j++) {
		sscanf(s+k," %f%n",eul+i*3+j,&l);
		k+=l;
	}
}
fclose(in);

// Ok, now set up the results matrix
pos=new EMData;
pos->setSize(DIM,np);
pos->zero();

// initialize positions
float *posd = pos->getData();

for (i=0; i<np*DIM; i++) posd[i]=0;		// points are independent in this case, so we can start in the middle
pos->doneData();

// Now optimize
for (i=0; i<np; i++) {
	float step=2.0;
	EMData *ps = pos->row(i);
	EMData *ds = mx.row(i);
	while (step>.001) {
		step=stepdown(ps,ds,refpos,step);
	}
	pos->setRow(ps,i);
}

// Write out the optimized results
posd=pos->getData();
FILE *out=fopen(argv[2],"w");
for (i=0; i<np; i++) {
	for (j=0; j<DIM; j++) {
		fprintf(out,"%1.4f\t",posd[i*DIM+j]);
	}
	
	float te=0;
	for (k=0; k<nr; k++) {
		float s=0;
		for (l=0; l<DIM; l++) s+=SQR(posd[i*DIM+l]-posd[k*DIM+l]);
		s=sqrt(s);
		te+=SQR(s-mx.valueAt(i,k)); 
	}
	te=sqrt(te);
	
	fprintf(out,"%f\t%f\t%f\t%f\n",te,eul[i*3],eul[i*3+1],eul[i*3+2]);
}
fclose(out);
}

// One step on one point towards optimum
float stepdown(EMData *ps,EMData *dst,EMData *refpos,float last) {
static float lastst=2.0;
if (last>0) lastst=last;
float step;
EMData *ds=dsdp(ps,dst,refpos);
EMData *dsc=ds->copy();
EMData *pos2;
float sigbest=sigma(ps,dst,refpos),stepbest=lastst*2.0;

for (step=lastst*2.0,dsc->multConst(lastst*2.0); step>.001; step*=.5,ds->multConst(.5)) {
	pos2=ps->copy(0,0);
	pos2->add(ds);
	float sig=sigma(pos2,dst,refpos);
	if (sig<sigbest) { sigbest=sig; stepbest=step; }
	delete pos2;
}
dsc->multConst(stepbest);
ps->add(dsc);
delete dsc;
delete ds;
return stepbest;
}

// calculate partial derivative of sigma vs current position 
EMData *dsdp(EMData *ps,EMData *ds,EMData *refpos) {
EMData *ret = new EMData;
ret->setSize(ps->xSize(),1,1);
int i,j;

for (j=0; j<ps->xSize(); j++) {
	float sig = 0,dst;
	for (i=0; i<ds->xSize(); i++) {
		dst=ps->dist(refpos,i);
		sig+=(ps->valueAt(j,i)-refpos->valueAt(j,i))*(dst-ds->valueAt(i,0))/dst;
	}
	ret->setValueAt(j,0,0,-sig);
}

return ret;
}

// Calculate 'error' in distances vs embedded distances
// ps - position of point to test
// ds - 'proper' distances to each reference point
// refpos - embedded reference points
float sigma(EMData *ps,EMData *ds,EMData *refpos) {
int i;

float sig = 0;
for (i=0; i<ds->xSize(); i++) {
	sig+=SQR(ps->dist(refpos,i)-ds->valueAt(i,0));
}
return sqrt(sig/ds->xSize());
}
